<?php

namespace Drupal\commerce_vado\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_order\Event\OrderEvent;
use Drupal\commerce_order\Event\OrderItemEvent;
use Drupal\commerce_order\Event\OrderEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\commerce_price\Price;

/**
 * Responds to preSave event to add an addon ot an order.
 */
class VadoEventSubscriber implements EventSubscriberInterface {

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Get subscribed events.
   *
   * @return array
   *   The subscribed events.
   */
  public static function getSubscribedEvents() {
    $events = [
      OrderEvents::ORDER_PRESAVE => ['orderPresaveEvent', -99],
      OrderEvents::ORDER_ITEM_UPDATE => ['orderItemUpdateEvent', -99],
      OrderEvents::ORDER_ITEM_DELETE => ['orderItemDeleteEvent', -99],
    ];
    return $events;
  }

  /**
   * Adds addon to the cart if it's not already there, or locks the quantity.
   */
  public function orderPresaveEvent(OrderEvent $event) {
    $order = $event->getOrder();
    $order_items = $order->getItems();
    $order_item_storage = $this->entityTypeManager->getStorage('commerce_order_item');

    foreach ($order_items as $order_item) {

      /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $parent_variation */
      $parent_variation = $order_item->getPurchasedEntity();

      $sync_quantity = $parent_variation->hasField('field_variation_addon_sync');
      if ($sync_quantity && !$parent_variation->get('field_variation_addon_sync')->isEmpty()) {
        $sync_quantity = $parent_variation->get('field_variation_addon_sync')->value;
      }

      //check if field_variation_addon exists and is not empty
      if (!$order_item->getData('commerce_vado_parent_order_item', FALSE) && $parent_variation->hasField('field_variation_addon') && !$parent_variation->get('field_variation_addon')->isEmpty()) {

        //create a discount_multiplier variable for field_variation_addon_multiplier field
        $discount_multiplier = 1;
        if ($parent_variation->hasField('field_variation_addon_multiplier') && !$parent_variation->get('field_variation_addon_multiplier')->isEmpty()) {
          $discount_multiplier = $parent_variation->get('field_variation_addon_multiplier')->getValue()[0]['value'];
        }
        //sets priceresolver order item price back to it's original price
        $order_item->setUnitPrice($parent_variation->getPrice(), TRUE);

        $child_order_items = $order_item->getData('commerce_vado_child_order_items', []);
        if (empty($child_order_items)) {
          foreach ($parent_variation->get('field_variation_addon')->referencedEntities() as $child_variation) {
            $child_order_item = $order_item_storage->create([
              'title' => $child_variation->getOrderItemTitle(),
              'type' => $child_variation->getOrderItemTypeId(),
              'unit_price' => $child_variation->getPrice()->multiply($discount_multiplier),
              'purchased_entity' => $child_variation->id(),
              'quantity' => $sync_quantity ? $order_item->getQuantity() : 1,
              'overridden_unit_price' => TRUE,
            ]);
            $child_order_item->setData('commerce_vado_synced_child_order_item', $sync_quantity);
            $child_order_item->setData('commerce_vado_parent_order_item', $order_item->id());
            $child_order_item->save();
            $child_order_items[] = $child_order_item->id();
            $order->addItem($child_order_item);
          }
          $order_item->setData('commerce_vado_child_order_items', $child_order_items);
        }
        $order_item->save();
      }
    }
  }

  public function orderItemUpdateEvent(OrderItemEvent $event) {
    $order_item_storage = $this->entityTypeManager->getStorage('commerce_order_item');
    $order_item = $event->getOrderItem();
    $child_order_item_ids = $order_item->getData('commerce_vado_child_order_items', []);
    $child_order_items = $order_item_storage->loadMultiple($child_order_item_ids);
    foreach ($child_order_items as $child_order_item) {
      if ($child_order_item->getData('commerce_vado_synced_child_order_item')) {
        $quantity = $order_item->getQuantity();
        $child_order_item->setQuantity($quantity);
        $child_order_item->save();
      }
    }
    if ($order_item->getOrderId()) {
      $order_item->getOrder()->setRefreshState('refresh_on_load');
    }
  }

  public function orderItemDeleteEvent(OrderItemEvent $event) {
    $order_item_storage = $this->entityTypeManager->getStorage('commerce_order_item');
    $order_item = $event->getOrderItem();
    $parent_variation = $order_item->getPurchasedEntity();
    $sync_quantity = $parent_variation->hasField('field_variation_addon_sync');
    if ($sync_quantity && !$parent_variation->get('field_variation_addon_sync')->isEmpty()) {
      $sync_quantity = $parent_variation->get('field_variation_addon_sync')->value;
    }
    if ($sync_quantity) {
      $child_order_item_ids = $order_item->getData('commerce_vado_child_order_items', []);
      if(!empty($child_order_item_ids)) {
        $child_order_items = $order_item_storage->loadMultiple($child_order_item_ids);
        $order_item_storage->delete($child_order_items);
      }
    }
  }
}
